/* vim: set textwidth=80 tabstop=4: */

/*
 * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2008 Michael Rasmussen and the Claws Mail Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "defs.h"

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "gtk/gtkutils.h"
#include "common/claws.h"
#include "common/version.h"
#include "plugin.h"
#include "mainwindow.h"
#include "utils.h"
#include "prefs.h"
#include "folder.h"
#include "foldersel.h"
#include "procmsg.h"
#include "libarchive_archive.h"
#include "archiver.h"

typedef struct _progress_widget progress_widget;
struct _progress_widget {
	GtkWidget*	progress_dialog;
	GtkWidget*	frame;
	GtkWidget*	vbox1;
	GtkWidget*	hbox1;
	GtkWidget*	add_label;
	GtkWidget*	file_label;
	GtkWidget*	progress;
	guint		position;
};

static progress_widget* progress;
static gboolean cancelled = FALSE;

static void progress_dialog_cb(GtkWidget* widget, gint action, gpointer data) {
	stop_archiving();
	cancelled = TRUE;
	archive_free_file_list();
	gtk_widget_destroy(widget);
}

static void create_progress_dialog(gchar* folder) {
	gchar* titel = g_strdup_printf("%s %s", _("Archiving"), folder);
	MainWindow* mainwin = mainwindow_get_mainwindow();

	progress->position = 0;
	progress->progress_dialog = gtk_dialog_new_with_buttons (
				titel,
				GTK_WINDOW(mainwin->window),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_STOCK_CANCEL,
				GTK_RESPONSE_CANCEL,
				NULL);

	g_signal_connect (
				progress->progress_dialog,
				"response",
				G_CALLBACK(progress_dialog_cb),
				NULL);

	progress->frame = gtk_frame_new(_("Press Cancel button to stop archiving"));
	gtk_frame_set_shadow_type(GTK_FRAME(progress->frame),
					GTK_SHADOW_ETCHED_OUT);
	gtk_container_set_border_width(GTK_CONTAINER(progress->frame), 4);
	gtk_container_add(GTK_CONTAINER(
				GTK_DIALOG(progress->progress_dialog)->vbox), progress->frame);

	progress->vbox1 = gtk_vbox_new (FALSE, 4);
	gtk_container_set_border_width (GTK_CONTAINER (progress->vbox1), 4);
	gtk_container_add(GTK_CONTAINER(progress->frame), progress->vbox1);
	
	progress->hbox1 = gtk_hbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(progress->hbox1), 8);
	gtk_box_pack_start(GTK_BOX(progress->vbox1),
					progress->hbox1, FALSE, FALSE, 0);

	progress->add_label = gtk_label_new(_("Adding"));
	gtk_box_pack_start(GTK_BOX(progress->hbox1),
					progress->add_label, FALSE, FALSE, 0);

	progress->file_label = gtk_label_new("");
	gtk_label_set_ellipsize(GTK_LABEL(progress->file_label),
					PANGO_ELLIPSIZE_START);
	gtk_box_pack_start(GTK_BOX(progress->hbox1),
					progress->file_label, TRUE, TRUE, 0);

	progress->hbox1 = gtk_hbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(progress->hbox1), 8);
	gtk_box_pack_start(GTK_BOX(progress->vbox1),
					progress->hbox1, FALSE, FALSE, 0);

	progress->progress = gtk_progress_bar_new();
	gtk_box_pack_start(GTK_BOX(progress->hbox1), 
					progress->progress, TRUE, TRUE, 0);

	gtk_window_set_default_size(GTK_WINDOW(progress->progress_dialog), 400, 80);
	gtk_widget_show_all(progress->progress_dialog);
}

static struct ArchivePage* init_archive_page() {
	struct ArchivePage* page = malloc(sizeof(struct ArchivePage));

	page->path = NULL;
	page->name = NULL;
	page->file = NULL;
	page->folder = NULL;
	page->response = FALSE;
	page->force_overwrite = FALSE;
	page->compress_methods = NULL;
	page->archive_formats = NULL;
	page->recursive = NULL;

	return page;
}

static void dispose_archive_page(struct ArchivePage* page) {
	if (page->path)
		g_free(page->path);
	page->path = NULL;
	if (page->name)
		g_free(page->name);
	page->name = NULL;
	if (page->compress_methods)
		g_slist_free(page->compress_methods);
	if (page->archive_formats)
		g_slist_free(page->archive_formats);
}

static gboolean uncommitted_entry_info(struct ArchivePage* page) {
	const gchar* path = gtk_entry_get_text(GTK_ENTRY(page->folder));
	const gchar* name = gtk_entry_get_text(GTK_ENTRY(page->file));
	
	if (! page->path && *path != '\0') {
		debug_print("page->path: (NULL) -> %s\n", path);
		page->path = g_strdup(path);
	}
	if (! page->name && *name != '\0') {
		page->force_overwrite = FALSE;
		debug_print("page->file: (NULL) -> %s\n", name);
		page->name = g_strdup(name);
	}
	
	return (page->path && page->name) ? TRUE : FALSE;
}

static gboolean valid_file_name(gchar* file) {
	int i;

	for (i = 0; INVALID_UNIX_CHARS[i] != '\0'; i++) {
		if (g_utf8_strchr(file, 
					g_utf8_strlen(file, -1), INVALID_UNIX_CHARS[i]))
			return FALSE;
	}
	return TRUE;
}

static COMPRESS_METHOD get_compress_method(GSList* btn) {
	const gchar* name;

	while (btn) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn->data))) {
			name = gtk_widget_get_name(GTK_WIDGET(btn->data));
			if (strcmp("ZIP", name) == 0) {
				debug_print("ZIP compression enabled\n");
				return ZIP;
			}
			else if (strcmp("BZIP", name) == 0) {
				debug_print("BZIP2 compression enabled\n");
				return BZIP2;
			}
/*			else if (strcmp("COMPRESS", name) == 0) {
				debug_print("COMPRESS compression enabled\n");
				return COMPRESS;
			}*/
			else if (strcmp("NONE", name) == 0) {
				debug_print("Compression disabled\n");
				return NO_COMPRESS;
			}
		}
		btn = g_slist_next(btn);
	}
	return NO_COMPRESS;
}

static ARCHIVE_FORMAT get_archive_format(GSList* btn) {
	const gchar* name;

	while (btn) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn->data))) {
			name = gtk_widget_get_name(GTK_WIDGET(btn->data));
			if (strcmp("TAR", name) == 0) {
				debug_print("TAR archive enabled\n");
				return TAR;
			}
			else if (strcmp("SHAR", name) == 0) {
				debug_print("SHAR archive enabled\n");
				return SHAR;
			}
			else if (strcmp("PAX", name) == 0) {
				debug_print("PAX archive enabled\n");
				return PAX;
			}
			else if (strcmp("CPIO", name) == 0) {
				debug_print("CPIO archive enabled\n");
				return CPIO;
			}
		}
		btn = g_slist_next(btn);
	}
	return NO_FORMAT;
}

static void walk_folder(struct ArchivePage* page, FolderItem* item,
				gboolean recursive) {
	FolderItem* child;
	GSList *msglist;
	GSList *cur;
	MsgInfo *msginfo;
	GNode* node;
	int count;

	if (recursive && ! cancelled) {
		debug_print("Scanning recursive\n");
		node = item->node->children;
		while(node && ! cancelled) {
			debug_print("Number of nodes: %d\n", g_node_n_children(node));
			if (node->data) {
				child = FOLDER_ITEM(node->data);
				debug_print("new node: %d messages\n", child->total_msgs);
				walk_folder(page, child, recursive);
			}
			node = node->next;
		}
	}
	if (! cancelled) {
		count = 0;
		page->files += item->total_msgs;
		msglist = folder_item_get_msg_list(item);
		for (cur = msglist; cur && ! cancelled; cur = cur->next) {
			msginfo = (MsgInfo *) cur->data;
			page->total_size += msginfo->size;
			gchar* file = folder_item_fetch_msg(item, msginfo->msgnum);
			debug_print("Processing: %s\n", file);
			if (file)
				archive_add_file(file);
			if (count % 250 == 0)
				GTK_EVENTS_FLUSH();
		}
		procmsg_msg_list_free(msglist);
	}
}

static gboolean archiver_save_files(struct ArchivePage* page) {
	GtkWidget* dialog;
	MainWindow* mainwin = mainwindow_get_mainwindow();
	FolderItem* item;
	COMPRESS_METHOD method;
	ARCHIVE_FORMAT format;
	gboolean recursive;

	if (page->path == NULL || page->name == NULL) {
		/* Test if page->file and page->folder has uncommitted information */
		if (! uncommitted_entry_info(page)) {
			dialog = gtk_message_dialog_new(
				GTK_WINDOW(mainwin->window),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_ERROR,
				GTK_BUTTONS_CLOSE,
				_("Folder and archive must be selected"));
			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);
			return FALSE;
		}
	}
	if (g_file_test(page->name, G_FILE_TEST_EXISTS) &&
				! page->force_overwrite) {
		dialog = gtk_message_dialog_new(
			GTK_WINDOW(mainwin->window),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_WARNING,
			GTK_BUTTONS_OK_CANCEL,
			_("%s: Exists. Continue anyway?"),
			page->name);
		int response = gtk_dialog_run(GTK_DIALOG (dialog));
		gtk_widget_destroy(dialog);
		if (response == GTK_RESPONSE_CANCEL) {
			return FALSE;
		}
	}
	if (! valid_file_name(page->name)) {
		dialog = gtk_message_dialog_new(
			GTK_WINDOW(mainwin->window),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_CLOSE,
			_("Not a valid file name:\n%s."),
			page->name);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return FALSE;
	}
	item = folder_find_item_from_identifier(page->path);
	if (! item) {
		dialog = gtk_message_dialog_new(
			GTK_WINDOW(mainwin->window),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_CLOSE,
			_("Not a valid claws-mail folder:\n%s."),
			page->path);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return FALSE;
	}
	page->files = 0;
	page->total_size = 0;
	recursive = gtk_toggle_button_get_active(
					GTK_TOGGLE_BUTTON(page->recursive));
	create_progress_dialog(page->path);
	GTK_EVENTS_FLUSH();
	walk_folder(page, item, recursive);
	if (cancelled)
		return FALSE;
	GSList* list = archive_get_file_list();
	if (page->files != g_slist_length(list)) {
		dialog = gtk_message_dialog_new(
				GTK_WINDOW(mainwin->window),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_WARNING,
				GTK_BUTTONS_OK_CANCEL,
				_("Adding files in folder failed\n"
				  "Files in folder: %d\n"
				  "Files in list:   %d\n"
				  "\nContinue anyway?"),
				page->files, g_slist_length(list));
		int response = gtk_dialog_run(GTK_DIALOG (dialog));
		gtk_widget_destroy(dialog);
		if (response == GTK_RESPONSE_CANCEL) {
			gtk_widget_destroy(progress->progress_dialog);
			archive_free_file_list();
			return FALSE;
		}
	}
	const gchar* res = NULL;
	method = get_compress_method(page->compress_methods);
	format = get_archive_format(page->archive_formats);
	if ((res = archive_create(page->name, list, method, format)) != NULL) {
		dialog = gtk_message_dialog_new(
			GTK_WINDOW(mainwin->window),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_CLOSE,
			"%s", res);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		archive_free_file_list();
		return FALSE;
	}
	archive_free_file_list();
	return TRUE;
}

static void entry_change_cb(GtkWidget* widget, gpointer data) {
	const gchar* name = gtk_widget_get_name(widget);
	struct ArchivePage* page = (struct ArchivePage *) data;

	if (strcmp("folder", name) == 0) {
		page->path = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
		debug_print("page->folder = %s\n", page->path);
	}
	else if (strcmp("file", name) == 0) {
		page->name = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
		page->force_overwrite = FALSE;
		debug_print("page->name = %s\n", page->name);
	}
}

static void show_result(struct ArchivePage* page) {
	
	enum {
		STRING1,
		STRING2,
		N_COLUMNS
	};

	struct stat st;
	GtkListStore* list;
	GtkTreeIter iter;
	GtkTreeView* view;
	GtkTreeViewColumn* header;
	GtkCellRenderer* renderer;
	GtkWidget* dialog;
	gchar* msg = NULL;
	gchar* method = NULL;
	gchar* format = NULL;

	MainWindow* mainwin = mainwindow_get_mainwindow();

	switch (get_compress_method(page->compress_methods)) {
		case ZIP:
			method = g_strdup("ZIP");
			break;
		case BZIP2:
			method = g_strdup("BZIP2");
			break;
/*		case COMPRESS:
			if (archive_write_set_compression_gzip(arch) != ARCHIVE_OK)
				return archive_error_string(arch);
			break;*/
		case NO_COMPRESS:
			method = g_strdup("No Compression");
			break;
	}
	
	switch (get_archive_format(page->archive_formats)) {
		case TAR:
			format = g_strdup("TAR");
			break;
		case SHAR:
			format = g_strdup("SHAR");
			break;
		case PAX:
			format = g_strdup("PAX");
			break;
		case CPIO:
			format = g_strdup("CPIO");
			break;
		case NO_FORMAT:
			format = g_strdup("NO FORMAT");
	}

	stat(page->name, &st);
	dialog = gtk_dialog_new_with_buttons(
			_("Archive result"),
			GTK_WINDOW(mainwin->window),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_STOCK_OK,
			GTK_RESPONSE_NONE,
			NULL);
	g_signal_connect_swapped(
				dialog,
				"response",
				G_CALLBACK(gtk_widget_destroy),
				dialog);

	list = gtk_list_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
	
	view = g_object_new(
				GTK_TYPE_TREE_VIEW,
				"model", list,
				"rules-hint", FALSE,
				"headers-clickable", FALSE,
				"reorderable", FALSE,
				"enable-search", FALSE,
				NULL);

	renderer = gtk_cell_renderer_text_new();

	header = gtk_tree_view_column_new_with_attributes(
				_("Attributes"), renderer, "text", STRING1, NULL);
	gtk_tree_view_append_column(view, header);

	header = gtk_tree_view_column_new_with_attributes(
				_("Values"), renderer, "text", STRING2, NULL);
	gtk_tree_view_append_column(view, header);

	gtk_container_add(
				GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), GTK_WIDGET(view));

	gtk_list_store_append(list, &iter);
	gtk_list_store_set(
				list, &iter,
				STRING1, _("Archive"),
				STRING2, page->name, -1);

	gtk_list_store_append(list, &iter);
	gtk_list_store_set(
				list, &iter,
				STRING1, _("Archive format"),
				STRING2, format, -1);
	g_free(format);

	gtk_list_store_append(list, &iter);
	gtk_list_store_set(
				list, &iter,
				STRING1, _("Compression method"),
				STRING2, method, -1);
	g_free(method);

	gtk_list_store_append(list, &iter);
	msg = g_strdup_printf("%d", page->files);
	gtk_list_store_set(
				list, &iter,
				STRING1, _("Number of files"),
				STRING2, msg, -1);
	g_free(msg);

	gtk_list_store_append(list, &iter);
	msg = g_strdup_printf("%d byte(s)", (guint) st.st_size);
	gtk_list_store_set(
				list, &iter,
				STRING1, _("Archive Size"),
				STRING2, msg, -1);
	g_free(msg);

	gtk_list_store_append(list, &iter);
	msg = g_strdup_printf("%d byte(s)", page->total_size);
	gtk_list_store_set(
				list, &iter,
				STRING1, _("Folder Size"),
				STRING2, msg, -1);
	g_free(msg);

	gtk_list_store_append(list, &iter);
	msg = g_strdup_printf("%d%%", 
					(guint)((st.st_size * 100) / page->total_size));
	gtk_list_store_set(
				list, &iter,
				STRING1, _("Compression level"),
				STRING2, msg, -1);
	g_free(msg);

	gtk_window_set_default_size(GTK_WINDOW(dialog), 320, 220);

	gtk_widget_show_all(dialog);
}

static void archiver_dialog_cb(GtkWidget* widget, gint action, gpointer data) {
	struct ArchivePage* page = (struct ArchivePage *) data;
	gboolean result = FALSE;

	switch (action) {
		case GTK_RESPONSE_ACCEPT:
			debug_print("User chose OK\n");
			page->response = TRUE;
			break;
		default:
			debug_print("User chose CANCEL\n");
			page->response = FALSE;
			archiver_gtk_done(page, widget);
	}
	debug_print("Settings:\nfolder: %s\nname: %s\naction: %d\n",
			page->path, page->name, page->response);
	if (page->response) {
		result = archiver_save_files(page);
		if (result && ! cancelled) {
			gtk_widget_destroy(progress->progress_dialog);
			show_result(page);
			gtk_widget_destroy(widget);
		}
		if (cancelled) {
			cancelled = FALSE;
		}
	}
}

static void foldersel_cb(GtkWidget *widget, gpointer data)
{
	FolderItem *item;
	gchar *item_id;
	gint newpos = 0;
	struct ArchivePage* page = (struct ArchivePage *) data;

	item = foldersel_folder_sel(NULL, FOLDER_SEL_MOVE, NULL, FALSE);
	if (item && (item_id = folder_item_get_identifier(item)) != NULL) {
		gtk_editable_delete_text(GTK_EDITABLE(page->folder), 0, -1);
		gtk_editable_insert_text(GTK_EDITABLE(page->folder),
					item_id, strlen(item_id), &newpos);
		page->path = g_strdup(item_id);
		g_free(item_id);
	}
	debug_print("Folder to archive: %s\n", 
				gtk_entry_get_text(GTK_ENTRY(page->folder)));
}

static void filesel_cb(GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	gchar* file;
	gint newpos = 0;
	const gchar* homedir;
	struct ArchivePage* page = (struct ArchivePage *) data;

	dialog = gtk_file_chooser_dialog_new(
		_("Select file name for archive [suffix should reflect archive like .tgz]"),
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
			NULL);
	homedir = g_getenv("HOME");
	if (!homedir)
		homedir = g_get_home_dir();

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), homedir);
	if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_APPLY) {
		file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		if (file) {
			gtk_editable_delete_text(GTK_EDITABLE(page->file), 0, -1);
			gtk_editable_insert_text(GTK_EDITABLE(page->file),
						file, strlen(file), &newpos);
			page->name = g_strdup(file);
			g_free(file);
			page->force_overwrite = TRUE;
		}
	}
	gtk_widget_destroy(dialog);
	debug_print("Name for archive: %s\n",
				gtk_entry_get_text(GTK_ENTRY(page->file)));
}

void set_progress_file_label(const gchar* file) {
	gtk_label_set_text(GTK_LABEL(progress->file_label), file);
}

void set_progress_print_all(guint fraction, guint total, guint step) {
	gchar* text_count;

	if ((fraction - progress->position) % step == 0) {
		debug_print("frac: %d, total: %d, step: %d, prog->pos: %d\n",
				fraction, total, step, progress->position);
		gtk_progress_bar_set_fraction(
					GTK_PROGRESS_BAR(progress->progress), 
					(total == 0) ? 0 : (gfloat)fraction / (gfloat)total);
		text_count = g_strdup_printf(_("%ld of %ld"), 
					(long) fraction, (long) total);
		gtk_progress_bar_set_text(
					GTK_PROGRESS_BAR(progress->progress), text_count);
		g_free(text_count);
		progress->position = fraction;
		GTK_EVENTS_FLUSH();
	}
}

void archiver_gtk_show() {
	GtkWidget* dialog;
	GtkWidget* frame;
	GtkWidget* vbox1;
	GtkWidget* hbox1;
	GtkWidget* folder_label;
	GtkWidget* folder_select;
	GtkWidget* file_label;
	GtkWidget* file_select;
	GtkWidget* zip_radio_btn;
	GtkWidget* bzip_radio_btn;
/*	GtkWidget* compress_radio_btn;*/
	GtkWidget* no_radio_btn;
	GtkWidget* shar_radio_btn;
	GtkWidget* pax_radio_btn;
	GtkWidget* cpio_radio_btn;
	GtkWidget* tar_radio_btn;
	GtkTooltips* tooltips;
	struct ArchivePage* page;

	progress = malloc(sizeof(*progress));
	MainWindow* mainwin = mainwindow_get_mainwindow();

	page = init_archive_page();

	tooltips = gtk_tooltips_new();

	dialog = gtk_dialog_new_with_buttons (
				_("Mail Archiver Dialog"),
				GTK_WINDOW(mainwin->window),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_STOCK_CANCEL,
				GTK_RESPONSE_CANCEL,
				GTK_STOCK_OK,
				GTK_RESPONSE_ACCEPT,
				NULL);

	g_signal_connect (
				dialog,
				"response",
				G_CALLBACK(archiver_dialog_cb),
				page);

	frame = gtk_frame_new(_("Enter Archiver arguments"));
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), frame);

	vbox1 = gtk_vbox_new (FALSE, 4);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), 4);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);
	
	hbox1 = gtk_hbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(hbox1), 4);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	folder_label = gtk_label_new(_("Folder to archive"));
	gtk_box_pack_start(GTK_BOX(hbox1), folder_label, FALSE, FALSE, 0);
	
	page->folder = gtk_entry_new();
	gtk_widget_set_name(page->folder, "folder");
	gtk_box_pack_start(GTK_BOX(hbox1), page->folder, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tooltips, page->folder,
			_("Folder which is root of archive"), NULL);

	folder_select = gtkut_get_browse_directory_btn(_("_Browse"));
	gtk_box_pack_start(GTK_BOX(hbox1), folder_select, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, folder_select,
			_("Click this button to select a folder which is to be root of archive"), NULL);

	hbox1 = gtk_hbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(hbox1), 4);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	file_label = gtk_label_new(_("Name for archive"));
	gtk_box_pack_start(GTK_BOX(hbox1), file_label, FALSE, FALSE, 0);
	
	page->file = gtk_entry_new();
	gtk_widget_set_name(page->file, "file");
	gtk_box_pack_start(GTK_BOX(hbox1), page->file, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tooltips, page->file,	_("Archive name"), NULL);

	file_select = gtkut_get_browse_directory_btn(_("_Select"));
	gtk_box_pack_start(GTK_BOX(hbox1), file_select, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, file_select,
			_("Click this button to select a name for your archive"), NULL);

	frame = gtk_frame_new(_("Choose compression"));
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);

	hbox1 = gtk_hbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(hbox1), 4);
	gtk_container_add(GTK_CONTAINER(frame), hbox1);

	zip_radio_btn = gtk_radio_button_new_with_mnemonic(NULL, "_ZIP");
	gtk_widget_set_name(zip_radio_btn, "ZIP");
	gtk_box_pack_start(GTK_BOX(hbox1), zip_radio_btn, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, zip_radio_btn,
			_("Choose this to use ZIP compression for your archive"), NULL);

	bzip_radio_btn = gtk_radio_button_new_with_mnemonic_from_widget(
					GTK_RADIO_BUTTON(zip_radio_btn), "BZIP_2");
	gtk_widget_set_name(bzip_radio_btn, "BZIP");
	gtk_box_pack_start(GTK_BOX(hbox1), bzip_radio_btn, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, bzip_radio_btn,
			_("Choose this to use BZIP2 compression for your archive"), NULL);
/*
	compress_radio_btn = gtk_radio_button_new_with_mnemonic_from_widget(
					GTK_RADIO_BUTTON(zip_radio_btn), "C_ompress");
	gtk_widget_set_name(compress_radio_btn, "COMPRESS");
	gtk_box_pack_start(GTK_BOX(hbox1), compress_radio_btn, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, compress_radio_btn,
		_("Choose this to use Compress compression for your archive"), NULL);
*/
	no_radio_btn = gtk_radio_button_new_with_mnemonic_from_widget(
					GTK_RADIO_BUTTON(zip_radio_btn), "_None");
	gtk_widget_set_name(no_radio_btn, "NONE");
	gtk_box_pack_start(GTK_BOX(hbox1), no_radio_btn, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, no_radio_btn,
		_("Choose this to disable compression for your archive"), NULL);

	page->compress_methods = 
			gtk_radio_button_get_group(GTK_RADIO_BUTTON(zip_radio_btn));

	frame = gtk_frame_new(_("Choose format"));
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);

	hbox1 = gtk_hbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(hbox1), 4);
	gtk_container_add(GTK_CONTAINER(frame), hbox1);

	tar_radio_btn = gtk_radio_button_new_with_mnemonic(NULL, "_TAR");
	gtk_widget_set_name(tar_radio_btn, "TAR");
	gtk_box_pack_start(GTK_BOX(hbox1), tar_radio_btn, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, tar_radio_btn,
			_("Choose this to use TAR as format for your archive"), NULL);

	shar_radio_btn = gtk_radio_button_new_with_mnemonic_from_widget(
					GTK_RADIO_BUTTON(tar_radio_btn), "S_HAR");
	gtk_widget_set_name(shar_radio_btn, "SHAR");
	gtk_box_pack_start(GTK_BOX(hbox1), shar_radio_btn, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, shar_radio_btn,
			_("Choose this to use SHAR as format for your archive"), NULL);

	cpio_radio_btn = gtk_radio_button_new_with_mnemonic_from_widget(
					GTK_RADIO_BUTTON(tar_radio_btn), "CP_IO");
	gtk_widget_set_name(cpio_radio_btn, "CPIO");
	gtk_box_pack_start(GTK_BOX(hbox1), cpio_radio_btn, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, cpio_radio_btn,
		_("Choose this to use CPIO as format for your archive"), NULL);

	pax_radio_btn = gtk_radio_button_new_with_mnemonic_from_widget(
					GTK_RADIO_BUTTON(tar_radio_btn), "PA_X");
	gtk_widget_set_name(pax_radio_btn, "PAX");
	gtk_box_pack_start(GTK_BOX(hbox1), pax_radio_btn, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, pax_radio_btn,
		_("Choose this to use PAX as format for your archive"), NULL);

	page->archive_formats = 
			gtk_radio_button_get_group(GTK_RADIO_BUTTON(tar_radio_btn));

	frame = gtk_frame_new(_("Choose recursion"));
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);

	hbox1 = gtk_hbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(hbox1), 4);
	gtk_container_add(GTK_CONTAINER(frame), hbox1);

	page->recursive = gtk_check_button_new_with_mnemonic("_Recursive");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page->recursive), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox1), page->recursive, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, page->recursive,
		_("Choose this option to add folders recursively"), NULL);
	
	g_signal_connect(G_OBJECT(folder_select), "clicked", 
			 G_CALLBACK(foldersel_cb), page);
	g_signal_connect(G_OBJECT(file_select), "clicked",
			 G_CALLBACK(filesel_cb), page);
	g_signal_connect(G_OBJECT(page->folder), "activate",
			 G_CALLBACK(entry_change_cb), page);
	g_signal_connect(G_OBJECT(page->file), "activate",
			 G_CALLBACK(entry_change_cb), page);

	gtk_widget_show_all(dialog);
}

void archiver_gtk_done(struct ArchivePage* page, GtkWidget* widget) {
	dispose_archive_page(page);
	free(progress);
	gtk_widget_destroy(widget);
}

